# Docker-Compose Backup Script

Opinionated backup script that reads all [docker-compose](https://docs.docker.com/compose/) projects (marked by a `docker-compose.yml` file) from a certain path, stops each project's services, backs up all attached volumes with [Restic](https://github.com/restic/restic) into a repository (one for each project) and starts the services again. It includes the docker-compose folders with the yml files.

## Docker-compose file prerequisites

**Important:** The volumes used in the projects need to be declared with `name: {}` in the volumes section to ensure they are created with the `com.docker.compose.volume` label that is used to locate volumes to be backed up. Otherwise they might not be backed up by the script.

Example:

```yml
volumes:
  my-volume: {}
```

## Configuration

The script can be configured either by command line parameters or by a configuration file. The possible parameters are as follows:

| command line | config file     | default                           | description                                                            |
| ------------ | --------------- | --------------------------------- | ---------------------------------------------------------------------- |
| `-b`         | `BACKUP_HOME`   | `/opt/docker/backups`             | The place where the backups should be located.                         |
| `-c`         |                 | `/opt/docker/.backupconfig`       | The config file.                                                       |
| `-h`         | `PROJECTS_HOME` | `/opt/docker/projects`            | The root location for the docker-compose projects.                     |
| `-k`         | `REMOTE_KEY`    |                                   | The key for the remote repository.                                     |
| `-l`         | `LOG_FILE_NAME` | `backups.log`                     | The name of the log file. It will be created in `BACKUP_HOME`          |
| `-p`         | `PASSWORD`      |                                   | The password for the restic backup repositories. **This is mandatory** |
| `-s`         | `REMOTE_SECRET` |                                   | The secret for the remote repository.                                  |
| `-u`         | `REMOTE_URL`    |                                   | The url for the remote repository.                                     |

## Failure notification

The `failure-notification.sh` script can inform a web address of failures during the backup. Currently, notification is only possible via slack webhooks. They can be configured as follows:

| command line | config file     | default                     | description                |
| ------------ | --------------- | --------------------------- | -------------------------- |
| `-c`         |                 | `/opt/docker/.backupconfig` | The config file.           |
| `-s`         | `SLACK_WEBHOOK` |                             | The slack webhook address. |

## Remote repository

It is possible to use the remote repository feature from restic for Amazon S3, Azure Blob Storage and Backblaze B2. Only one provider can be configured at a time. The configuration is as follows:

| Config Parameter | Backend   | Usage                             |
| ---------------- | --------- | --------------------------------- |
| `REMOTE_KEY`     | Amazon S3 | IAM Access Key                    |
| `REMOTE_SECRET`  | Amazon S3 | IAM Secret Access Key             |
| `REMOTE_URL`     | Amazon S3 | s3:s3.amazonaws.com/[bucket name] |
| `REMOTE_KEY`     | Azure     | Storage account name              |
| `REMOTE_SECRET`  | Azure     | Key 1                             |
| `REMOTE_URL`     | Azure     | azure:[blob container name]       |
| `REMOTE_KEY`     | B2        | keyId                             |
| `REMOTE_SECRET`  | B2        | applicationKey                    |
| `REMOTE_URL`     | B2        | b2:[bucket name]                  |

## Restoring

Currently, there is no restore functionality included in the scripts, you have to [use the restic container directly](https://restic.readthedocs.io/en/stable/050_restore.html):

_Example:_ Restore the latest local snapshot

```
sudo docker run --rm --name restic \
    -e RESTIC_PASSWORD=[Backup password] \
    -v [absolute path on parent]:/restore \
    restic/restic \
    -r [Repository path] \
    restore latest --target /restore
```

_Example:_ Restore the latest snapshot, stored on Azure

```
sudo docker run --rm --name restic \
    -e RESTIC_PASSWORD=[Backup password] \
    -e AZURE_ACCOUNT_NAME=[Account name] \
    -e AZURE_ACCOUNT_KEY=[Account key] \
    -v [absolute path on parent]:/restore \
    restic/restic \
    -r azure:[Blob container name]:/[Repository path] \
    restore latest --target /restore
```