#!/bin/bash

#*************************************************************************
#*************************************************************************
# Failure notification script; see README.md for details/instructions
#*************************************************************************
#*************************************************************************

CONFIG_FILE=/opt/docker/.backupconfig

LOG_FILE=/opt/docker/backups/backups.log
SLACK_WEBHOOK=

##############
# Command line parameter initialization
##############

while getopts "c:s:" opt; do
	case ${opt} in
	c)
		CONFIG_FILE=$OPTARG
		;;
	s)
		FROM_ADDRESS=$OPTARG
		;;
	*) ;;
	esac
done
shift $((OPTIND - 1))

##############
# Read config file
##############

if [ -f "$CONFIG_FILE" ]; then
	while IFS="=" read -r key value; do
		case "$key" in
		'#'*) ;;
		*)
			if [ -n "$key" ]; then
				eval "${key}=\"${value}\""
			fi
			;;
		esac
	done <"$CONFIG_FILE"
fi

##############
# Exit if no mail address or log file
##############

if [ -z "$LOG_FILE" ]; then
	exit 1
fi

##############
# Parse log file
##############

message=
today=$(date +%F)
todaysEntries=$(grep -r "$today" "$LOG_FILE")

if [ -z "$todaysEntries" ]; then
	message="The backup process was not completed sucessfully."
else
	lastEntry=${todaysEntries[${#todaysEntries[@]} - 1]}
	if [[ $lastEntry == *"error"* ]]; then
		message="The backup process was completed with errors."
	fi
fi

##############
# Notify
##############

if [ -n "$message" ]; then
	if [ -n "$SLACK_WEBHOOK" ]; then
		docker run --rm \
			--name slack-notify \
			-e SLACK_WEBHOOK="$SLACK_WEBHOOK" \
			-e SLACK_TITLE="Warning!" \
			-e SLACK_COLOR="#FF5722" \
			-e SLACK_MESSAGE="$(printf "%s\nTodays log:\n\n%s" "$message" "$todaysEntries")" \
			technosophos/slack-notify
	fi
fi
